"""JARVIS database models."""
from datetime import datetime, timezone
from typing import Any, List

import marshmallow as ma
from umongo import Document, EmbeddedDocument, fields

from jarvis_core.db import CTC2_INST, JARVIS_INST
from jarvis_core.db.models.actions import Ban, Kick, Mute, Unban, Warning
from jarvis_core.db.models.modlog import Action, Modlog, Note
from jarvis_core.db.models.reddit import (
    Redditor,
    RedditorFollow,
    Subreddit,
    SubredditFollow,
)
from jarvis_core.db.models.twitter import TwitterAccount, TwitterFollow


def get_now() -> datetime:
    """Get proper timestamp."""
    return datetime.now(tz=timezone.utc)


class RawField(fields.BaseField, ma.fields.Raw):
    pass


@JARVIS_INST.register
class Autopurge(Document):
    guild: int = fields.IntegerField(required=True)
    channel: int = fields.IntegerField(required=True)
    delay: int = fields.IntegerField(default=30)
    admin: int = fields.IntegerField(required=True)
    created_at: datetime = fields.DateTimeField(default=datetime.now)


@JARVIS_INST.register
class Autoreact(Document):
    guild: int = fields.IntegerField(required=True)
    channel: int = fields.IntegerField(required=True)
    reactions: List[str] = fields.ListField(fields.StringField())
    admin: int = fields.IntegerField(required=True)
    thread: bool = fields.BooleanField(default=True)
    created_at: datetime = fields.DateTimeField(default=datetime.now)


@JARVIS_INST.register
class Config(Document):
    """Config database object."""

    key: str = fields.StringField(required=True)
    value: Any = RawField(required=True)


@CTC2_INST.register
class Guess(Document):
    """Guess database object."""

    correct: bool = fields.BooleanField(default=False)
    guess: str = fields.StringField(required=True)
    user: int = fields.IntegerField(required=True)


@JARVIS_INST.register
class Permission(EmbeddedDocument):
    """Embedded Permissions document."""

    id: int = fields.IntegerField(required=True)
    allow: int = fields.IntegerField(default=0)
    deny: int = fields.IntegerField(default=0)


@JARVIS_INST.register
class Lock(Document):
    """Lock database object."""

    active: bool = fields.BooleanField(default=True)
    admin: int = fields.IntegerField(required=True)
    channel: int = fields.IntegerField(required=True)
    duration: int = fields.IntegerField(default=10)
    guild: int = fields.IntegerField(required=True)
    reason: str = fields.StringField(required=True)
    original_perms: Permission = fields.EmbeddedField(Permission, required=False)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class Lockdown(Document):
    """Lockdown database object."""

    active: bool = fields.BooleanField(default=True)
    admin: int = fields.IntegerField(required=True)
    duration: int = fields.IntegerField(default=10)
    guild: int = fields.IntegerField(required=True)
    reason: str = fields.StringField(required=True)
    original_perms: int = fields.IntegerField(required=True)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class Purge(Document):
    """Purge database object."""

    admin: int = fields.IntegerField(required=True)
    channel: int = fields.IntegerField(required=True)
    guild: int = fields.IntegerField(required=True)
    count: int = fields.IntegerField(default=10)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class Reminder(Document):
    """Reminder database object."""

    active: bool = fields.BooleanField(default=True)
    user: int = fields.IntegerField(required=True)
    guild: int = fields.IntegerField(required=True)
    channel: int = fields.IntegerField(required=True)
    message: str = fields.StringField(required=True)
    remind_at: datetime = fields.DateTimeField(required=True)
    created_at: datetime = fields.DateTimeField(default=get_now)
    private: bool = fields.BooleanField(default=False)


@JARVIS_INST.register
class Rolegiver(Document):
    """Rolegiver database object."""

    guild: int = fields.IntegerField(required=True)
    roles: List[int] = fields.ListField(fields.IntegerField())


@JARVIS_INST.register
class Bypass(EmbeddedDocument):
    """Roleping bypass embedded object."""

    users: List[int] = fields.ListField(fields.IntegerField())
    roles: List[int] = fields.ListField(fields.IntegerField())


@JARVIS_INST.register
class Roleping(Document):
    """Roleping database object."""

    active: bool = fields.BooleanField(default=True)
    role: int = fields.IntegerField(required=True)
    guild: int = fields.IntegerField(required=True)
    admin: int = fields.IntegerField(required=True)
    bypass: Bypass = fields.EmbeddedField(Bypass)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class Setting(Document):
    """Setting database object."""

    guild: int = fields.IntegerField(required=True)
    setting: str = fields.StringField(required=True)
    value: Any = RawField()


@JARVIS_INST.register
class Star(Document):
    """Star database object."""

    active: bool = fields.BooleanField(default=True)
    index: int = fields.IntegerField(required=True)
    message: int = fields.IntegerField(required=True)
    channel: int = fields.IntegerField(required=True)
    starboard: int = fields.IntegerField(required=True)
    guild: int = fields.IntegerField(required=True)
    admin: int = fields.IntegerField(required=True)
    star: int = fields.IntegerField(required=True)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class Starboard(Document):
    """Starboard database object."""

    channel: int = fields.IntegerField(required=True)
    guild: int = fields.IntegerField(required=True)
    admin: int = fields.IntegerField(required=True)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class Tag(Document):
    """Tag database object."""

    creator: int = fields.IntegerField(required=True)
    created_at: datetime = fields.DateTimeField(default=get_now)
    name: str = fields.StringField(required=True)
    content: str = fields.StringField(required=True)


@JARVIS_INST.register
class Temprole(Document):
    """Temporary role object."""

    guild: int = fields.IntegerField(required=True)
    user: int = fields.IntegerField(required=True)
    role: int = fields.IntegerField(required=True)
    admin: int = fields.IntegerField(required=True)
    expires_at: datetime = fields.DateTimeField(required=True)
    created_at: datetime = fields.DateTimeField(default=get_now)


@JARVIS_INST.register
class UserSetting(Document):
    """User Setting object."""

    user: int = fields.IntegerField(required=True)
    type: str = fields.StringField(required=True)
    setting: str = fields.StringField(required=True)
    value: Any = RawField()

    class Meta:
        collection_name = "usersetting"
